﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace XamDataGridDynamicColumns
{
    public class Period : INotifyPropertyChanged, IDataErrorInfo
    {
        public string Title { get; set; }
        private string _dataError;
        private int _hours;
        private Dictionary<string, string> _dataErrors = new Dictionary<string, string>();


        public event PropertyChangedEventHandler PropertyChanged;

        public int Hours
        {
            get { return _hours; }
            set
            {
                if (_hours == value)
                    return;
                if (value < 0 || value > 1200 || value % 2 == 1)
                    throw new Exception("The value must be an odd value between 0 and 1200");
                _hours = value;

                this.ValidateHours(false);

                RaisePropertyChanged(nameof(Hours));
            }
        }

        private void ValidateHours(bool raiseNotification)
        {
            string error = null;
            if (_hours == null)
                error = "No Value";

            this.SetFieldDataError("Period 1", error, raiseNotification);
            this.SetFieldDataError("Period 2", error, raiseNotification);
            this.SetFieldDataError("Period 3", error, raiseNotification);
            this.SetFieldDataError("Period 4", error, raiseNotification);
        }

        #region GetFieldDataError

        public string GetFieldDataError(string fieldName)
        {
            string error;
            _dataErrors.TryGetValue(fieldName, out error);

            return error;
        }

        #endregion // GetFieldDataError

        #region GetRecordDataError

        public string GetRecordDataError()
        {
            return _dataError;
        }

        #endregion // GetRecordDataError

        #region HasDataError

        public bool HasDataError()
        {
            return !string.IsNullOrEmpty(_dataError)
                || null != _dataErrors && _dataErrors.Count > 0;
        }

        #endregion // HasDataError

        #region SetFieldDataError

        public void SetFieldDataError(string fieldName, string error, bool raiseNotification)
        {
            if (string.IsNullOrEmpty(error))
            {
                _dataErrors.Remove(fieldName);

                // If there are no field errors anymore, reset the record error as well.
                if (0 == _dataErrors.Count)
                    this.SetRecordDataError(null, true);
            }
            else
            {
                _dataErrors[fieldName] = error;

                this.SetRecordDataError(error, true);
            }

            if (raiseNotification)
            { }
            //this.RaisePropertyChanged(fieldName);
        }

        #endregion // SetFieldDataError

        #region SetRecordDataError

        public void SetRecordDataError(string error, bool raiseNotification)
        {
            if (_dataError != error)
            {
                _dataError = error;

                if (raiseNotification)
                { }
                //this.RaisePropertyChanged("");
            }
        }

        #endregion // SetRecordDataError

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get
            {
                return _dataError;
            }
        }

        string IDataErrorInfo.this[string fieldName]
        {
            get
            {
                return this.GetFieldDataError(fieldName);
            }
        }

        #endregion

        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
